package com.nifty.homepage2.jyube.hanabi;

import android.graphics.Canvas;
import android.graphics.Paint;

public class Hanabi {
	public static float GRAVITY = 0.8f;
	public static long PASSAGE_MAX = 1000*2;
	public static int POWER_MAX = 360;

	private float x, y, radius;
	private float dx, dy;
	private int count;
	private float decay;
	private Paint color;

	public Hanabi(float x, float y, float dx, float dy, float decay, Paint color) {
		this.x = x;
		this.y = y;
		this.dx = dx;
		this.dy = dy;
		this.count = 9 + (int)(Math.random() * 2);
		this.radius = 2;
		this.decay = decay;
		this.color = color;
	}
	
	public float getX() {
		return x;
	}

	public float getY() {
		return y;
	}

	public float getDx() {
		return dx;
	}

	public float getDy() {
		return dy;
	}

	public Paint getColor() {
		return color;
	}

	public boolean update() {
		x += dx;
		y += dy;
		dx *= GRAVITY;
		dy *= GRAVITY;
		return (--count <= 0);
	}
	
	public void draw(Canvas canvas) {
		canvas.drawCircle(x, y, radius, color);
	}

	public FireBall createFireBall(float angle) {
		FireBall fb = new FireBall();

		fb.setColor(color);
		fb.setPosition(x, y);
		fb.setRadius(10);
		fb.setVecter(10, angle);
		fb.addVecter(dx, dy);
		fb.setDecay(decay);
		
		return fb;
	}

	public static class Seed {
		private Screen.Position pos;
		private Screen.Position mov;
		private float power;
		private long time;

		public Seed(Screen.Position pos) {
			this.mov = this.pos = pos;
			this.power = 1.0f;
			this.time = System.currentTimeMillis();
		}

		public void move(Screen.Position mov) {
			float dx = mov.getX() - this.mov.getX();
			float dy = mov.getY() - this.mov.getY();
			this.power += Math.sqrt(dx * dx + dy * dy);
			this.mov = mov;
		}

		public Screen.Position getPosition() {
			return pos;
		}
		public float getPower() {
			return power;
		}
		public long getTime() {
			return time;
		}
		
		public Hanabi createHanabi(Screen.Position sp) {
			// 射出方向
			float dx = (sp.getX() - pos.getX()) / 2.0f;
			float dy = (sp.getY() - pos.getY()) / 2.0f;

			// 減衰率
			long passage = System.currentTimeMillis() - time;
			float decay = ((passage > PASSAGE_MAX)?
					0: PASSAGE_MAX - passage) * 0.1f / PASSAGE_MAX + 0.005f;

			// 色
			float h = (power > POWER_MAX)? POWER_MAX: power;
			Paint color = new Color.Hsv(h, 1.0f, 1.0f).toColor();

			return new Hanabi(pos.getX(), pos.getY(), dx, dy, decay, color);
		}
	}
}
